"""Supply chain domain — warehouses, suppliers, purchasing, inventory, and logistics."""

from sqllocks_spindle.domains.supply_chain.supply_chain import SupplyChainDomain

__all__ = ["SupplyChainDomain"]
